@@ -25,8 +25,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <string.h>
-#include <glib/gerror.h>
-#include <glib/gtypes.h>
+#include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gst/gst.h>
@@ -60,7 +59,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define DEFAULT_AUDIO_PROFILE_NAME "cdlossy"
 
 /* Element names */
-#define FILE_SINK "gnomevfssink"
+#define FILE_SINK "giosink"
 
 struct SjExtractorPrivate {
   /** The current audio profile */
@@ -308,8 +307,9 @@ error_cb (GstBus *bus, GstMessage *message, gpointer user_data)
   g_error_free (error);
 }
 
+#if 0
 /**
- * Callback from the gnomevfssink to say that its about to overwrite a file.
+ * Callback from the giosink to say that its about to overwrite a file.
  * For now, Just Say Yes.  If this API will stay in 0.9, then rewrite
  * SjExtractor.
  */
@@ -318,6 +318,7 @@ just_say_yes (GstElement *element, gpointer filename, gpointer user_data)
 {
   return TRUE;
 }
+#endif
 
 static void
 build_pipeline (SjExtractor *extractor)
@@ -379,7 +380,9 @@ build_pipeline (SjExtractor *extractor)
                  _("Could not create GStreamer file output"));
     return;
   }
+#if 0
   g_signal_connect (G_OBJECT (priv->filesink), "allow-overwrite", G_CALLBACK (just_say_yes), extractor);
+#endif
 
   /* Add the elements to the pipeline */
   gst_bin_add_many (GST_BIN (priv->pipeline), priv->cdsrc, priv->queue, priv->encoder, priv->filesink, NULL);
@@ -465,17 +468,19 @@ sj_extractor_set_paranoia (SjExtractor *extractor, const int paranoia_mode)
 }
 
 void
-sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, const char* url, GError **error)
+sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, GFile *file, GError **error)
 {
+  GParamSpec *spec;
   GstStateChangeReturn state_ret;
   SjExtractorPrivate *priv;
   GstIterator *iter;
   GstTagSetter *tagger;
   gboolean done;
+  char *uri;
 
   g_return_if_fail (SJ_IS_EXTRACTOR (extractor));
 
-  g_return_if_fail (url != NULL);
+  g_return_if_fail (file != NULL);
   g_return_if_fail (track != NULL);
 
   priv = extractor->priv;
@@ -491,11 +496,16 @@ sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, c
   }
 
   /* Need to do this, as playback will have locked the read speed to 2x previously */
-  g_object_set (G_OBJECT (priv->cdsrc), "read-speed", G_MAXINT, NULL);
+  spec = g_object_class_find_property (G_OBJECT_GET_CLASS (priv->cdsrc), "read-speed");
+  if (spec && spec->value_type == G_TYPE_INT) {
+    g_object_set (G_OBJECT (priv->cdsrc), "read-speed", ((GParamSpecInt*)spec)->maximum, NULL);
+  }
 
   /* Set the output filename */
   gst_element_set_state (priv->filesink, GST_STATE_NULL);
-  g_object_set (G_OBJECT (priv->filesink), "location", url, NULL);
+  uri = g_file_get_uri (file);
+  g_object_set (G_OBJECT (priv->filesink), "location", uri, NULL);
+  g_free (uri);
 
   /* Set the metadata */
   iter = gst_bin_iterate_all_by_interface (GST_BIN (priv->pipeline), GST_TYPE_TAG_SETTER);
@@ -514,7 +524,6 @@ sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, c
                                GST_TAG_DURATION, track->duration * GST_SECOND,
                                NULL);
 
-#ifdef GST_TAG_MUSICBRAINZ_ALBUMID
       if (track->album->album_id != NULL && strcmp (track->album->album_id, "") != 0) {
         gst_tag_setter_add_tags (tagger,
                             GST_TAG_MERGE_APPEND,
@@ -542,16 +551,21 @@ sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, c
       if (track->artist_sortname != NULL && strcmp (track->artist_sortname, "") != 0) {
         gst_tag_setter_add_tags (tagger,
                             GST_TAG_MERGE_APPEND,
-                            GST_TAG_MUSICBRAINZ_SORTNAME, track->artist_sortname,
+                            GST_TAG_ARTIST_SORTNAME, track->artist_sortname,
                             NULL);
       }
-#endif
 
       if (track->album->genre != NULL && strcmp (track->album->genre, "") != 0) {
-        gst_tag_setter_add_tags (tagger,
-                                 GST_TAG_MERGE_APPEND,
-                                 GST_TAG_GENRE, track->album->genre,
-                                 NULL);
+        char **values, **l;
+        values = g_strsplit (track->album->genre, ",", 0);
+        for (l = values; *l; l++) {
+          g_strstrip (*l);
+          gst_tag_setter_add_tags (tagger,
+                                   GST_TAG_MERGE_APPEND,
+                                   GST_TAG_GENRE, *l,
+                                   NULL);
+        }
+        g_strfreev (values);
       }
       if (track->album->release_date) {
         gst_tag_setter_add_tags (tagger,
@@ -559,10 +573,16 @@ sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, c
                                  GST_TAG_DATE, track->album->release_date,
                                  NULL);
       }
+      if (track->album->disc_number > 0) {
+        gst_tag_setter_add_tags (tagger,
+                                 GST_TAG_MERGE_APPEND,
+                                 GST_TAG_ALBUM_VOLUME_NUMBER, track->album->disc_number,
+                                 NULL);
+      }
       gst_object_unref (tagger);
       break;
     case GST_ITERATOR_RESYNC:
-      // TODO?
+      /* TODO? */
       g_warning ("Got GST_ITERATOR_RESYNC, not sure what to do");
       gst_iterator_resync (iter);
       break;
